/*

=head1 NAME

XPointerHttpRequest.js - XPointerHttpRequest is a JavaScript library to request XML fragments using XPointer

=head1 SYNOPSIS

   # you'll need to do this under Mozilla/Gecko (but not Safari) 
   # netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");

   var doc = "http://127.0.0.1/recipes/beans/baked_beans.xml";

   var xpath = new XPointerXPath ();
   xpath.add_namespace("e","http://www.eatdrinkfeelgood.org/1.1/ns");
   xpath.range("e:ing");

   var xpointer = new XPointerHttpRequest (url);
   xpointer.range(xpath);
   xpointer.accept("multipart/mixed");
 
   var res = xpointer.send();

   if (res.is_success()) {
      alert(res.err() + ":" + res.errstr());
      return false;
   }

   if (res.content_type() == "application/xml") {
      // res is an XMLDocument object
   }

   else {
     // res is a string, which we process locally
   }

=head1 DESCRIPTION

XPointerHttpRequest is a JavaScript library to request XML fragments using 
XPointer and the HTTP 1.1 Range and Accept headers, as described in the paper :
A Semantic Web Resource Protocol: XPointer and HTTP.

=cut

*/

/*

=head1 XPointerHttpRequest CLASS METHODS

=head2 new XPointerHttpRequest (url)

I<url> is a string representing the URL of the document you are
requesting.

Returns an I<XPointerHttpRequest> object.

=cut

*/

function XPointerHttpRequest (url) {
    this.inheritFrom = Object;
    this.inheritFrom();
    
    this._url    = url;
    this._range  = null;
    this._accept = null;
}

/*

=head1 XPointerHttpRequest OBJECT METHODS

=head2 xpointer.range(query)

Set the search criteria used to prune the document you are requesting.

String. 

(Note that the I<toString> method for XPointer(XPath|RDQL) objects are
set up to return a valid string representation so they may be passed,
as is, to this method.)

=cut

*/

XPointerHttpRequest.prototype.range = function (query) {
    this._range = query;
}

/*

=head2 xpointer.accept(list)

Set the list of acceptable content-types that may be returned by the
remote server.

String.

=cut

*/

XPointerHttpRequest.prototype.accept = function (list) {
    this._accept = list;
}

/*

=head2 xpointer.send(callback)

Send the request.

I<callback> is an optional function reference to be executed if you
want the request to be processed asynchronously. If defined, the method
will always return true and it's up to you to process the HTTP response.

Otherwise, returns a I<XPointerResult> or I<XPointerError> object.

=cut

*/

XPointerHttpRequest.prototype.send = function (callback) {

    var ua = this._useragent();

    if (! ua) {
	return new XPointerError(500,"Failed to create XMLHttpRequest object");
    }

    var async   = (callback) ? true      : false;
    var content = (async)    ? undefined : null;

    ua.open("GET",url,async);

    ua.setRequestHeader('Range',this._range);

    if (this._accept) {
	ua.setRequestHeader('Accept',this._accept);
    }

    if (callback) {
	ua.onreadystatechange = callback;
    }

    try {
	ua.send(content);
    }

    catch (e) {
	return new XPointerError(500,e);
    }

    if (async) {
	return true;
    }

    if (ua.status != 206) {
	return new XPointerError(ua.status,ua.statusText);
    }

    var regexp = new RegExp();
    regexp.compile("^(?:application|text)/(?:rdf+)?xml$");

    var ctype = ua.getResponseHeader("Content-type");

    // alert("type"+ctype);
    // alert(ua.responseText);

    if (regexp.test(ctype)) {
	return new XPointerResult(ctype,ua.responseXML);
    }

    else {
	return new XPointerResult(ctype,ua.responseText);
    }
}

XPointerHttpRequest.prototype._useragent = function () {

    var xmlhttp = null;

    /*@cc_on @*/
    /*@if (@_jscript_version >= 5)

    try {
       xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
    } 

    catch (e) {
       try {
          xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
       } 

       catch (E) {
          return null;
       }
    }

    @end @*/

    if ((! xmlhttp) && (typeof(XMLHttpRequest) != 'undefined')) {
	xmlhttp = new XMLHttpRequest();
    }

    return xmlhttp;
}

/*

=head1 XPointerXPath CLASS METHODS

=head2 new XPointerXPath()

Returns a I<XPointerXPath> object.

=cut

*/

function XPointerXPath () {
    this.inheritFrom = XPointerScheme;
    this.inheritFrom();
}

/*

=head1 XPointerXPath OBJECT METHODS

A I<XPointerXPath> object's toString() method is set up to return a
a complete XPointer statement, based on the input passed to the I<range>
and I<add_namespace> method. For example :

  var xpath = new XPointerXPath ();
  xpath.add_namespace("e","http://www.eatdrinkfeelgood.org/1.1/ns");
  xpath.range("e:ing");

  alert(xpath);

  # prints 'xmlns(e=http://www.eatdrinkfeelgood.org/1.1/ns)xpointer(e:ing)'

=head2 xpath.add_namespace(prefix,namespaceurl)

Add a new namespace to your XPath query.

=head2 xpath.range(query)

Set the XPath sent to the remote server.

String.

=cut

*/

XPointerXPath.prototype.toString = function () {
    var xmlns = new Array ();

    for (prefix in this._ns) {
        var url = this._ns[prefix];
        xmlns.push("xmlns(" + prefix + "=" + url + ")");
    }

    return xmlns.join("") + "xpointer(" + this._range + ")";
}

/*

=head1 XPointerRDQL CLASS METHODS

=head2 new XPointerRDQL()

Returns a I<XPointerRDQL> object.

=cut

*/

function XPointerRDQL () {
    this.inheritFrom = XPointerScheme;
    this.inheritFrom();
}

/*

=head1 XPointerRDQL OBJECT METHODS

A I<XPointerRDQL> object's toString() method is set up to return a
a complete RDQL statement, based on the input passed to the I<range>
and I<add_namespace> method. For example :

  var rdql = new XPointerRDQL ();
  rdql.add_namespace("rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#");
  rdql.add_namespace("rss","http://purl.org/rss/1.0/");

  rdql.range("SELECT ?title, ?link WHERE "        + 
             "(?item, <rdf:type>,   <rss:item>)," +
	     "(?item, <rss::title>, ?title),"     +
	     "(?item, <rss::link>,  ?link)");

  alert(rdql);

  # prints SELECT ?title, ?link
  #        WHERE
  #        (?item, <rdf:type>, <rss:item>),
  #        (?item, <rss::title>, ?title),
  #        (?item, <rss::link>, ?link)
  #        USING
  #        rdf for <http://www.w3.org/1999/02/22-rdf-syntax-ns#>,
  #        rss for <http://purl.org/rss/1.0/>;

=head2 rdql.add_namespace(prefix,namespaceurl)

Add a new namespace to your RDQL query.

=head2 rdql.range(query)

Set the RDQL sent to the remote server.

String.

=cut

*/

XPointerRDQL.prototype.toString = function () {
    var xmlns = new Array ();

    for (var i = 0; i < this._ns.length; i++) {
        var prefix = this._ns[i];
        var url    = this._ns[prefix];

        xmlns.push(prefix + " for " + "<" + url + ">");
    }

    if (! xmlns.length) {
	return this._range;
    }

    return this._range + " USING " + xmlns.join(" ");
}

function XPointerScheme () {
    this.inheritFrom = Object;
    this.inheritFrom();

    this._range = null;
    this._ns    = new Object;

    this.range = function (q) {
       this._range = q;
    }

    this.add_namespace = function (prefix,url) {
	this._ns[prefix] = url;
    }
}

function XPointerResult (ctype,data) {
    this.inheritFrom = XPointerResponse;
    this.inheritFrom();

    this._ctype = ctype;
    this._data  = data;

    this._success = true;
}

/*

=head1 XPointerResult OBJECT METHODS

A I<XPointerResults> object's toString() method is set up to return
either an I<XMLDocument> object if its content-type is either :

=over 4

=item * B<text/xml>

=item * B<application/xml>

=item * B<application/rdf+xml>

=back

Otherwise it return a string, containing the content of the server's
response.

=cut

*/

/* Dunno - simply returning this._data
   in toString() causes weird errors when
   data is an XMLDocument object */

XPointerResult.prototype.toString = function () {
    return this.data();
}

XPointerResult.prototype.data = function () {
    return this._data;
}

/*

=head2 result.is_success()

Returns true or false.

=cut

=head2 result.content_type()

Returns a string containing the value of the HTTP Content-type header.

=cut

*/

XPointerResult.prototype.content_type = function() {
    return this._ctype;
}

/*

=head1 XPointerError OBJECT METHODS

=cut

*/

function XPointerError (code,str) {
    this.inheritFrom = XPointerResponse;
    this.inheritFrom();

    this._err     = code;
    this._errstr  = str;
    this._success = false;
}

/*

=head2 error.is_success()

Returns true or false.

=head2 error.err ()

Returns the code associated with the error.

=cut

*/

XPointerError.prototype.err = function() {
    return this._err;
}

/* 

=head2 error.errstr ()

Returns the optional message associated with the error.

=cut

*/

XPointerError.prototype.errstr = function() {
    return this._errstr;
}

function XPointerResponse () {
    this.inheritFrom = Object;
    this.inheritFrom();

    this._success = null;

    this.is_success = function () {
	return this._success;
    }
}

/*

=head1 VERSION

1.0

=head1 DATE

$Date: 2004/11/18 20:12:00 $

=head1 AUTHOR

Aaron Straup Cope E<lt>http://www.aaronstraupcope.comE<gt>

=head1 SEE ALSO

http://www.mindswap.org/papers/swrp-iswc04.pdf

http://www.w3.org/TR/WD-xptr

http://www.w3.org/Submission/2004/SUBM-RDQL-20040109/

=head1 LICENSE

Copyright (c) 2004 Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as the Perl Artistic License.

=cut

$Id: xpointerhttprequest.js,v 1.2 2004/11/18 20:12:00 asc Exp $

*/
